import datetime
import os

from combo.data.models import Page
from django.conf import settings
from django.http import HttpResponseRedirect, JsonResponse
from django.urls import reverse
from django.views.generic.base import TemplateView
from emissions.models import Diffusion, Emission, Episode, NewsItem

import panikweb.views


class Home(panikweb.views.Home):
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['emissions'] = list(Emission.objects.filter(archived=False).order_by('creation_timestamp'))
        # force detour-vers-le-futur to go first
        detour = [x for x in context['emissions'] if x.slug == 'detour-vers-le-futur']
        context['emissions'] = detour + [x for x in context['emissions'] if x.slug != 'detour-vers-le-futur']
        context['newsitems'] = (
            NewsItem.objects.exclude(date__gt=datetime.date.today())
            .exclude(expiration_date__lt=datetime.date.today())
            .order_by('-date')[:3]
        )
        context['extra_pages'] = Page.objects.filter(exclude_from_navigation=False)
        return context

    def get(request, *args, **kwargs):
        emissions = Emission.objects.filter(archived=False)
        if len(emissions) == 1:
            return HttpResponseRedirect(reverse('emission-view', kwargs={'slug': emissions[0].slug}))
        return super().get(request, *args, **kwargs)


home = Home.as_view()


class EmissionDetailView(panikweb.views.EmissionDetailView):
    def get_emission_context(self, emission, episode_ids=None):
        context = super().get_emission_context(emission, episode_ids)

        if emission.archived:
            current_year = emission.creation_timestamp.replace(month=5, day=1)
        else:
            current_year = datetime.datetime.now().replace(month=5, day=1).replace(year=2021)

        episodes_queryset = Episode.objects.select_related()
        if episode_ids is not None:
            episodes_queryset = episodes_queryset.filter(id__in=episode_ids)
        else:
            episodes_queryset = episodes_queryset.filter(emission=emission)

        context['all_episodes'] = (
            episodes_queryset.filter(creation_timestamp__gt=current_year)
            .extra(
                select={
                    'first_diffusion': 'emissions_diffusion.datetime',
                },
                select_params=(False, True),
                where=[
                    '''datetime = (SELECT MIN(datetime)
                                                FROM emissions_diffusion
                                               WHERE episode_id = emissions_episode.id
                                             )'''
                ],
                tables=['emissions_diffusion'],
            )
            .order_by('first_diffusion')
            .distinct()
        )

        return context


emission = EmissionDetailView.as_view()


class ArchivesView(TemplateView):
    template_name = 'archives.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['diffusions'] = (
            Diffusion.objects.filter(episode__soundfile__isnull=False)
            .distinct()
            .order_by('-datetime')
            .select_related('episode', 'episode__emission')
        )
        return context


archives = ArchivesView.as_view()


def onair(request):
    if datetime.date.today() >= datetime.date(2022, 8, 1):
        return JsonResponse(
            {'data': {'emission': {'title': 'On se revoit en 2023, découvrez les podcasts !', 'url': '/'}}}
        )

    playing_txt = os.path.join(settings.MEDIA_ROOT, 'playing.txt')
    if os.path.exists(playing_txt):
        track_title = open(playing_txt).read().strip()
        if len(track_title) > 5:
            return JsonResponse({'data': {'emission': {'title': track_title, 'url': '/'}}})

    return JsonResponse({'data': {'emission': {'title': '(on arrive bientôt…)', 'url': '/'}}})
